/*!
*	\addtogroup Modulo5
*	\{
*/
/*!
*	\addtogroup TestModulo5 Specifiche per il Test
*	\{
*/

#include "LPC_M5_Include.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAXOCC 100				// Numero max occ di un elemento nel DB
#define MAXITER 101				// Numero max iterazioni ricerca. Deve essere > MAXOCC
#define NUMFIELDS 10			// Numero campi nei DB di prova
#define MAX_NUM_IN 100			// num max elementi da ricercare presenti nel DB
#define MAX_NUM_NOTIN 100		// num max elementi da ricercare non presenti nel DB
#define NUM_DEL_ADD 4			// numero cancellazioni/inserimenti per test incremental reindex
#define NUM_REC_SEARCH_DB 100	// numero di record nel DB per il test della ricerca

static int handleBuildIndex = -1;	// handle DB test Build
static int handleSearch = -1;		// handle DB test Search
static int handleReindex[NUM_DEL_ADD+1] = { -1, -1, -1, -1, -1 }; // handles DBs test Incremental Reindex

static FIELD_DATA_t FieldData[NUMFIELDS];			// pe rla lettura dei dati sui campi nell'header
static FIELD_VALUE_t FieldValueBuild[NUMFIELDS];	// valori su cui creare indici (tutti i campi del DB)
static FIELD_VALUE_t FieldValueNoIndex;				// Fieldd Value indice non creato
static FIELD_VALUE_t FieldValueSearchIn[MAX_NUM_IN];	// valori da ricercare - presenti nel DB
static int NumOccurrences[MAX_NUM_IN];					// numero di occorrenze dei valori da cercare
static int OccurrenceIn[MAX_NUM_IN][MAXOCC];				// posizione record dei valori da cercare
static FIELD_VALUE_t FieldValueSearchNotIn[MAX_NUM_NOTIN];	// valori da ricercare - non presenti nel DB
static int StartSearchNotIn[MAX_NUM_NOTIN];				// inizio ricerca per valori non nel DB
static FIELD_VALUE_t FieldValueDelAdd[NUM_DEL_ADD][NUMFIELDS];		// valori per reindex: cancellazione/inserimento
static int PositionDelAdd[NUM_DEL_ADD];					// posizione cancellazione/inserimento
static char *Nome, *Sesso, *Comune, *Provincia, *Telefono, *Indirizzo, *Studente;
static unsigned long *Data;
static double *Eta;
static double *Reddito;
static int NumIn = 0;
static int NumNotIn = 0;

void TestBuildIndexFile(void);
void TestSearchRecordPosition(void);
void TestIncrementalReindex(void);
void TestMisto(void);  		//!< Funzione di test misto per tutte le funzioni
void Result(char *Func);	//!< Funzione di calcolo e stampa risultati
int ConfrontaFiles(char *File1, char *File2); //!< Funzione di confronto files
int CleanUpReindex(int type); // Clean up indici per prova reindex
int Startup(int reset);	//!< Funzione di inizializzazione
void Quit(void);
int ConfrontaValues(FIELD_VALUE_t *FieldValue, FIELD_VALUE_t *FieldValueRef, int NumValues);

// Vettori per gli id dei test, il check e i pesi relativi
#define MAX_TEST 100
typedef struct {
	int testid;
	int weight;
	char desc[256];
	int passed;
} TEST_t;
		
static int numtest;
static TEST_t test[MAX_TEST];

/*!
*	\brief Test per le funzioni di interfaccia del Modulo 5
*
* Programma di test per il Modulo 5
*
* Ogni test e' identificato da un numero cosi' composto:\n
* \p xyynn \n
*  x	Id del modulo\n
* yy	Id della funzione\n
* nn	Id del test\n
*
* Ogni test ha anche un peso che viene indicato nella stampa dei risultati.
* I test di funzionalita' hanno i pesi piu' elevati.
* In questo test viene verificato che l'errore ritornato sia effettivamente 
* quello indicato dalla funzione.
*
* La funzione di test puo' essere chiamata anche da linea di comando 
* con: ./TestM5 opzione startup
* Dove opzione e' il numero dell'opzione scelta e startup e' 1 o 0 a seconda
* che si voglia reinizializzare o meno il file dbf.
*
* Vengono effettuati i seguenti controlli: 
 *
 * - Creazione Indice (BuildIndexFile()):
 *   - Handle errato o non aperto
 *   - Stringa nome campo non valida o vuota
 *   - Campo non valido
 *   - Verifica funzionalita' 
 *
 * - Ricerca Record (SearchRecordPosition()): 
 *   - Handle non aperto o errato
 *   - Stringa nome campo non valida o vuota
 *   - Campo non valido
 *   - Campo non indicizzato
 *   - Valore campo non valido
 *   - Verifica funzionalita'
 *
 * - Inserimento/Cancellazione Record (IncrementalReindex()):
 *   - Handle non aperto o errato
 *   - Stringa nome campo non valida o vuota
 *   - Campo non valido
 *   - Valore record di inizio ricerca non valido
 *   - Verifica funzionalita'
 *
 * - Misto
 *   **** NON IMPLEMENTATO ****
 *
 ****************************************************/

int main(int argc, char **argv)
{
	char scelta[10];
	int sel, reset, ret;

	printf("Programma di Test per il Modulo 5\n\n");

	// Vedo se mi e' stato chiesta l'inizializzazione completa
	if (argc == 3)
		reset = atoi(argv[2]);
	else
		if (argc < 2)
			reset = 1;
		else
			reset = 0;

	// Pulizia iniziale e preparazione dell'handle e allocazione
				
	if ((ret = Startup(reset)) < 0)
	{
		printf("TEST: Errore apertura database:[%d]\n", ret);
		printf("Exiting....\n");
		return(-1);
	}
	
	for(sel = 0;;)
	{
		if (argc < 2)
		{
			printf("\t1. Test funzione BuildIndexFile\n");
			printf("\t2. Test funzione SearchRecordPosition\n");
			printf("\t3. Test funzione IncrementalReindex\n");
			printf("\t4. Test funzione Misto [NON IMPLEMENTATO]\n");
			printf("\t5. Test completo e valutazione\n");
			printf("\n\t0. Uscita\n");
			printf("\n\nScelta: ");

			(void) fgets(scelta, sizeof(scelta), stdin);
			sel = atoi(scelta);
		}
		else // Faccio solo 1 giro
		{
			if (sel)
				sel = 0;
			else
				sel = atoi(argv[1]);
		}

		Result(NULL);
		switch(sel)
		{
		case 0:
			Quit();
			if (argc < 2)
				printf("Bye!\n\n");
			return(0);
		case 1:
			TestBuildIndexFile();
			Result("BUILDINDEX");	
			break;
		case 2:
			TestSearchRecordPosition();
			Result("SEARCHPOS");	
			break;
		case 3:
			TestIncrementalReindex();
			Result("REINDEX");	
			break;
		case 4:
			TestMisto();
			Result("MISTO");	
			break;
		case 5:
		{
			// Devo ripulire prima di ricominciare
			int ret = Startup(1);
			if (ret < 0)
			{
				printf("TEST: Errore apertura database:[%d]\n", ret);
				printf("Exiting....\n");
				return(-1);
			}
			TestBuildIndexFile();
			TestSearchRecordPosition();
			TestIncrementalReindex();
			TestMisto();
			Result("FINALE");
			break;
		}
		default:
			printf("Scelta errata...\n");
			break;;
		}
	}
}

void TestBuildIndexFile()
{
	int ret= 0;

	printf("Test BUILDINDEX\n");

// Test sull'handle
	test[numtest].testid = 50101;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "BUILDINDEX: Handle errato");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  BuildIndexFile(handleBuildIndex + 33, "Reddito");	// un fieldname corretto
	printf("Ritorno della BuildIndexFile: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul fieldname nullo
	test[numtest].testid = 50102;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "BUILDINDEX: fieldname nullo");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  BuildIndexFile(handleBuildIndex, NULL);
	printf("Ritorno della BuildIndexFile: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul fieldname stringa vuota
	test[numtest].testid = 50103;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "BUILDINDEX: fieldname stringa vuota");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  BuildIndexFile(handleBuildIndex, "");
	printf("Ritorno della BuildIndexFile: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul fieldname non esistente
	test[numtest].testid = 50104;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "BUILDINDEX: fieldname non esistente");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  BuildIndexFile(handleBuildIndex, "zia agata"); // nome di field non esistente
	printf("Ritorno della BuildIndexFile: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Verifica funzionalita' (1)
	test[numtest].testid = 50105;
	test[numtest].weight = 20;
	strcpy(test[numtest].desc, "BUILDINDEX: Funzionalita'(1)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	int i;
	for(i = 0; i < NUMFIELDS; i++)
	{
		ret = BuildIndexFile(handleBuildIndex, FieldValueBuild[i].Name);
		printf("Ritorno della BuildIndexFile: [%d]\n", ret);
		if (ret != LPC_OK)
			break;
		// controllo che il file sia uguale a quello giusto
		char indice1[100] = { 0 };
		char indice2[100] = { 0 };
		sprintf(indice1, "Prova4.%s.ndx", FieldValueBuild[i].Name);
		sprintf(indice2, "ProvaTest4.%s.ndx", FieldValueBuild[i].Name);
		if (ConfrontaFiles(indice1, indice2))
		{
			ret = -1;
			printf("Funzionalita BuildIndex: I file '%s' e '%s' sono diversi\n", indice1, indice2);
			break;
		}
	}
	if (ret == LPC_OK)
		test[numtest].passed = 1;
	numtest++;
}

void TestSearchRecordPosition()
{
	int ret= 0;
	int i, j, OccurrenceFound[MAXOCC];
	FIELD_VALUE_t FieldValue;
	long recnum = 1;

	printf("Test SEARCHPOS\n");

// Test sull'handle
	test[numtest].testid = 50201;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "SEARCHPOS: Handle errato");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	FieldValue = FieldValueSearchIn[0];
	ret =  SearchRecordPosition(handleSearch + 33, &FieldValue, &recnum);
	printf("Ritorno della SearchRecordPosition: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul FieldValue  (1)
	test[numtest].testid = 50202;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "SEARCHPOS: FieldValue nullo");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret =  SearchRecordPosition(handleSearch, NULL, &recnum);
	printf("Ritorno della SearchRecordPosition: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul FieldValue  (2)
	test[numtest].testid = 50203;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "SEARCHPOS: FieldValue.Name nullo");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	FieldValue = FieldValueSearchIn[0];
	FieldValue.Name[0] = 0;
	ret =  SearchRecordPosition(handleSearch, &FieldValue, &recnum);
	printf("Ritorno della SearchRecordPosition: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul FieldValue  (3)
	test[numtest].testid = 50204;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "SEARCHPOS: FieldValue.Name non esistente");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	FieldValue = FieldValueSearchIn[0];
	strcpy(FieldValue.Name, "zia Agata");  // campo non esistente
	ret =  SearchRecordPosition(handleSearch, &FieldValue, &recnum);
	printf("Ritorno della SearchRecordPosition: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul FieldValue  (4)
	test[numtest].testid = 50205;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "SEARCHPOS: FieldValue.value NULL");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	FieldValue = FieldValueSearchIn[0];
	FieldValue.value = NULL;
	ret =  SearchRecordPosition(handleSearch, &FieldValue, &recnum);
	printf("Ritorno della SearchRecordPosition: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul recnum  (1)
	test[numtest].testid = 50206;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "SEARCHPOS: recnum nullo");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	FieldValue = FieldValueSearchIn[0];
	ret =  SearchRecordPosition(handleSearch, &FieldValue, NULL);
	printf("Ritorno della SearchRecordPosition: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul recnum  (2)
	test[numtest].testid = 50207;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "SEARCHPOS: recnum negativo");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	FieldValue = FieldValueSearchIn[0];
	recnum = -1;
	ret =  SearchRecordPosition(handleSearch, &FieldValue, &recnum);
	printf("Ritorno della SearchRecordPosition: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

	// Test campo non indicizzato  (1)
	test[numtest].testid = 50208;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "SEARCHPOS: campo non indicizzato");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	recnum = 0;
	FieldValue = FieldValueNoIndex;
	ret =  SearchRecordPosition(handleSearch, &FieldValue, &recnum);
	printf("Ritorno della SearchRecordPosition: [%d]\n", ret);
	if (ret == LPC_FIELD_NOT_INDEXED)
		test[numtest].passed = 1;
	numtest++;
	
	// Verifica funzionalita' (1):  ricerca record prensenti nel DB
	test[numtest].testid = 50209;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "SEARCHPOS: Funzionalita' (1), ricerca record presenti nel DB");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);	
	for(i=0; i<NumIn; i++)
	{
		// cerca tutti i valori a partire dall'inizio del DB
		recnum = 0;
		for (j = 0;
			 recnum <= NUM_REC_SEARCH_DB && j < MAXITER &&
			 (ret = SearchRecordPosition(handleSearch, &FieldValueSearchIn[i], &recnum)) == LPC_OK;
			 j++) {
			printf("Ritorno della SearchRecordPosition: [%d]\n", ret);
			OccurrenceFound[j] = recnum;
		}
		printf("Ritorno della SearchRecordPosition: [%d]\n", ret);
		// controllo condizione di uscita e che il numero di occorrenze sia giusto
		if (ret != LPC_NOT_FOUND || j != NumOccurrences[i]) {
			ret = -999;
			break;
		}
		// controllo che le posizioni siano quelle giuste
		while (--j >= 0) {
			if (OccurrenceIn[i][j] != OccurrenceFound[j]) {
				ret = -999;
				break;
			}
		}
	}
	if (ret == LPC_NOT_FOUND)
		test[numtest].passed = 1;
	numtest++;
		
	// Verifica funzionalita' (2):  ricerca record non prensenti nel DB
	test[numtest].testid = 50210;
	test[numtest].weight = 10;
	strcpy(test[numtest].desc, "SEARCHPOS: Funzionalita' (2), ricerca record presenti nel DB");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);	
	for(i=0; i < NumNotIn; i++) {
		// cerca valori non presenti a partire dall'inizio del DB
		recnum = 0;
		ret = SearchRecordPosition(handleSearch, &FieldValueSearchNotIn[i], &recnum);
		if (ret != LPC_NOT_FOUND)
			break;
	}	
	if (ret == LPC_NOT_FOUND)
		test[numtest].passed = 1;
	numtest++;
		
	return;
}

void TestIncrementalReindex(void)
{
	int ret= 0;
	FIELD_VALUE_t FieldValue = FieldValueDelAdd[0][0];
	long numrec = 1;
	int i, j, isInsertion = ADD_RECORD;
	int handle = handleReindex[0];

	printf("Test REINDEX\n");

//extern int IncrementalReindex(int handle, FIELD_VALUE_t *FieldValue, int isInsertion, long numrec);
// Test sull'handle
	test[numtest].testid = 50301;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "REINDEX: Handle errato");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret = IncrementalReindex(handle + 44, &FieldValue, isInsertion, numrec);
	printf("Ritorno della IncrementalReindex: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul FieldValue  (1)
	test[numtest].testid = 50302;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "REINDEX: FieldValue NULL (1)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	ret = IncrementalReindex(handle, NULL, isInsertion, numrec);
	printf("Ritorno della IncrementalReindex: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul FieldValue  (2)
	test[numtest].testid = 50303;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "REINDEX: FieldValue->Name null");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	FieldValue.Name[0] = 0;
	ret = IncrementalReindex(handle, &FieldValue, isInsertion, numrec);
	printf("Ritorno della IncrementalReindex: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul FieldValue  (3)
	test[numtest].testid = 50304;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "REINDEX: FieldValue->Name non esistente");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	strcpy(FieldValue.Name, "zia Agata");
	ret = IncrementalReindex(handle, &FieldValue, isInsertion, numrec);
	printf("Ritorno della IncrementalReindex: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul isInsertion  (1)
	test[numtest].testid = 50305;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "REINDEX: isInsertion non valido");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	FieldValue = FieldValueDelAdd[0][0];
	isInsertion = 4;
	ret = IncrementalReindex(handle, &FieldValue, isInsertion, numrec);
	printf("Ritorno della IncrementalReindex: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul isInsertion  (2)
	test[numtest].testid = 50306;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "REINDEX: isInsertion non valido");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	isInsertion = -1;
	ret = IncrementalReindex(handle, &FieldValue, isInsertion, numrec);
	printf("Ritorno della IncrementalReindex: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul numrec  (1)
	test[numtest].testid = 50307;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "REINDEX: numrec non valido (-1)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	isInsertion = ADD_RECORD;
	numrec = -1;
	ret = IncrementalReindex(handle, &FieldValue, isInsertion, numrec);
	printf("Ritorno della IncrementalReindex: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

// Test sul numrec  (2)
	test[numtest].testid = 50308;
	test[numtest].weight = 3;
	strcpy(test[numtest].desc, "REINDEX: numrec non valido (9000)");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	numrec = 9000;
	ret = IncrementalReindex(handle, &FieldValue, isInsertion, numrec);
	printf("Ritorno della IncrementalReindex: [%d]\n", ret);
	if (ret == LPC_BAD_ARG)
		test[numtest].passed = 1;
	numtest++;

	// Verifica funzionalita'  (1):  inserimento
	test[numtest].testid = 50309;
	test[numtest].weight = 15;
	strcpy(test[numtest].desc, "REINDEX: funzionalita' (1) - inserimento");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	CleanUpReindex(ADD_RECORD);
	for (i = 0; i < NUM_DEL_ADD; i++) {
		for (j = 0; j < NUMFIELDS; j++) {
			FieldValue = FieldValueDelAdd[i][j];
			numrec = PositionDelAdd[i];
			ret = IncrementalReindex(handleReindex[i], &FieldValue, ADD_RECORD, numrec);
			printf("Ritorno della IncrementalReindex: [%d]\n", ret);
			if (ret != LPC_OK)
				break;
			char indice1[100] = { 0 };
			char indice2[100] = { 0 };
			sprintf(indice1, "Prova6%d.%s.ndx", i, FieldValue.Name);
			sprintf(indice2, "ProvaTest6%d.%s.ndx", i, FieldValue.Name);
			if (ConfrontaFiles(indice1, indice2)) {
				ret = -999;
				printf("Funzionalita REINDEX inserimento: I file '%s' e '%s' sono diversi\n", indice1, indice2);
				break;
			}
		}
		if (ret != LPC_OK)
			break;
	}
	if (ret == LPC_OK)
		test[numtest].passed = 1;
	numtest++;
	
// Verifica funzionalita'  (2):  cancellazione
	test[numtest].testid = 50310;
	test[numtest].weight = 15;
	strcpy(test[numtest].desc, "REINDEX: funzionalita' (2) - cancellazione");
	printf("Test:[%d] Peso:[%d] - [%s]\n", test[numtest].testid, test[numtest].weight,test[numtest].desc);
	CleanUpReindex(DEL_RECORD);
	for (i = 0; i < NUM_DEL_ADD; i++) {
		for (j = 0; j < NUMFIELDS; j++) {
			FieldValue = FieldValueDelAdd[i][j];
			numrec = PositionDelAdd[i];
			ret = IncrementalReindex(handleReindex[i+1], &FieldValue, DEL_RECORD, numrec);
			printf("Ritorno della IncrementalReindex: [%d]\n", ret);
			if (ret != LPC_OK)
				break;
			char indice1[100] = { 0 };
			char indice2[100] = { 0 };
			sprintf(indice1, "Prova6%d.%s.ndx", i+1, FieldValue.Name);
			sprintf(indice2, "ProvaTest6%d.%s.ndx", i+1, FieldValue.Name);
			if (ConfrontaFiles(indice1, indice2)) {
				ret = -999;
				printf("Funzionalita REINDEX cancellazione: I file '%s' e '%s' sono diversi\n", indice1, indice2);
				break;
			}
		}
		if (ret != LPC_OK)
			break;
	}
	if (ret == LPC_OK)
		test[numtest].passed = 1;
	numtest++;

	return;
}



void TestMisto()
{
	int j, ret = 0, handle1;
	DBF_INFO_t DBFInfo;
	char c;

	printf("Test MISTO\n");

	printf("NON IMPLEMENTATO!\n");
}

/*********************************
* RESULT
* Stampa i risultati. Se il parametro e' NULL allora resetta i valori
**********************************/
void Result(char *Func)
{
	int i, quanti, totval;

	// Svuoto il vettore
	if (!Func)
	{
		for (i = 0; i < MAX_TEST; i++)
			memset(&test[i], 0, sizeof(TEST_t));
		numtest = 0;
		return;
	}

	// Stampo gli ID
	printf("\n");
	printf("ID:   ");
	for (i = 0; i < numtest; i++)
	{
		printf("%6d", test[i].testid);
	}

	// Stampo i pesi e prendo il valore totale
	printf("\n");
	printf("PESO: ");
	totval = 0;
	for (i = 0; i < numtest; i++)
	{
		printf("%6d", test[i].weight);
		totval += test[i].weight;
	}

	// Stampo i risultati
	printf("\n");
	printf("PASS: ");
	quanti = 0;
	for (i = 0; i < numtest; i++)
	{
		printf("%6d", test[i].passed);
		if (test[i].passed)
			quanti += test[i].weight;
	}

	printf("\n\nFine Test [%s]: Numtests:[%d] Totvalue:[%d] Passed:[%d] Score:[%3.2f]\n\n", Func, numtest, totval, quanti, (double) (quanti*100/((double)totval)));
}


/*************************
* CONFRONTAFILES
**************************/
#define FILELEN 50000
int ConfrontaFiles(char *File1, char *File2)
{
	int len, len1, len2;
	char buf1[FILELEN], buf2[FILELEN];
	FILE *fp1, *fp2;

	if ((fp1 = fopen(File1, "r")) == NULL)
		return -1;
	len1 = fread(buf1, sizeof(char), FILELEN, fp1);
	fclose(fp1);

	if ((fp2 = fopen(File2, "r")) == NULL)
		return -1;
	len2 = fread(buf2, sizeof(char), FILELEN, fp2);
	fclose(fp2);
	if (len1 != len2)
		return(-1);

// non ce n'e' bisogno
	// Azzero il timestamp
//	len = DBF_NAME_LENGTH+1+sizeof(short)+sizeof(long)+sizeof(short)+sizeof(short);
//	memset(buf1+len, 0, sizeof(long));
//	memset(buf2+len, 0, sizeof(long));

	// Confronto i buffer
	if (memcmp(buf1, buf2, len1))
		return(-1);

	return 0;
}

/*******************
* CLEAN UP REINDEX
********************/
// Pulizia dei file di indice e dei db usati nella Reindex
// Reinizializza gli indici e i db per l'esecuzione del tipo
// di reindex specificato nel parametro ricevuto in input
int CleanUpReindex(int type) {
	int i, j;
	char cmd[256] = { 0 };
	char indice1[256] = { 0 };
	char indice2[256] = { 0 };

	system("rm -f Prova6?.*.ndx");

	switch (type) {
		case ADD_RECORD:
			for (i = 0; i < NUMFIELDS; i++)
				for (j = 0; j < NUM_DEL_ADD; j++) {	
					sprintf(indice1, "ProvaTest6%d.%s.ndx", j+1, FieldValueDelAdd[j][i].Name);
					sprintf(indice2, "Prova6%d.%s.ndx", j, FieldValueDelAdd[j][i].Name);
					sprintf(cmd, "cp -f %s %s", indice1, indice2);
					system(cmd);
				}
			break;
		case DEL_RECORD:
			for (i = 0; i < NUMFIELDS; i++)
				for (j = 0; j < NUM_DEL_ADD; j++)
				{	
					sprintf(indice1, "ProvaTest6%d.%s.ndx", j, FieldValueDelAdd[j][i].Name);
					sprintf(indice2, "Prova6%d.%s.ndx", j+1, FieldValueDelAdd[j][i].Name);
					sprintf(cmd, "cp -f %s %s", indice1, indice2);
					system(cmd);
				}
			break;
		default:
			break;
	}
	
	return 0;
}

/*******************
* STARTUP
********************/
int Startup(int reset)
{
	int i, j, idx, len, ret;
	int *Occ[MAX_NUM_IN];

	if (IsDBFHandle(handleBuildIndex) == LPC_OK)
		CloseDatabaseFile(handleBuildIndex);
	if (IsDBFHandle(handleSearch) == LPC_OK)
		CloseDatabaseFile(handleSearch);
	for (i = 0; i <= NUM_DEL_ADD; i++)
		if (IsDBFHandle(handleReindex[i]) == LPC_OK)
			CloseDatabaseFile(handleReindex[i]);

// Se reset completo cancello, ricopio dal template e apro il DBF
	if (reset)
	{
		system("rm -f Prova?.{dbf,\\$\\$\\$}");
		system("rm -f Prova?.*.ndx");
		system("cp -f ProvaTest4.dbf Prova4.dbf");
		system("cp -f ProvaTest5.dbf Prova5.dbf");
		system("cp -f ProvaTest5.Nome.ndx Prova5.Nome.ndx");
		system("cp -f ProvaTest5.Studente.ndx Prova5.Studente.ndx");
		system("cp -f ProvaTest5.Data.ndx Prova5.Data.ndx");
		system("cp -f ProvaTest5.Eta.ndx Prova5.Eta.ndx");
		system("rm -f Prova6?.{dbf,\\$\\$\\$}");
		system("rm -f Prova6?.*.ndx");
		system("cp -f ProvaTest60.dbf Prova60.dbf");
		system("cp -f ProvaTest61.dbf Prova61.dbf");
		system("cp -f ProvaTest62.dbf Prova62.dbf");
		system("cp -f ProvaTest63.dbf Prova63.dbf");
		system("cp -f ProvaTest64.dbf Prova64.dbf");
	}
	handleBuildIndex = OpenDatabaseFile("Prova4");
	if (handleBuildIndex < 0)
		return(handleBuildIndex);
	handleSearch = OpenDatabaseFile("Prova5");
	if (handleSearch < 0)
		return(handleSearch);
	for (i = 0; i <= NUM_DEL_ADD; i++) {
		char str[128];
		sprintf(str, "Prova6%d", i);
		handleReindex[i] = OpenDatabaseFile(str);
		if (handleReindex[i] < 0)
			return(handleReindex[i]);
	}
	
//
// Valori per Build
//
	ret = GetFieldInfo(handleBuildIndex, 0, &FieldData[0]);
	if (ret != LPC_OK)
		return(ret);
	
	for (i = 0; i < NUMFIELDS; i++) {
		// 0123456789
		// 3692581470
		idx = ((i+1)*3)%NUMFIELDS;

		// FieldValueBuild
		strcpy(FieldValueBuild[i].Name, FieldData[idx].Name);
		len = FieldData[idx].FieldLen;
		if (FieldData[idx].FieldType == CHARACTER)
			len++;
		FieldValueBuild[i].value = calloc(len, sizeof(char));
	}

	// Riempio FieldValueBuild
	Nome = (char *) FieldValueBuild[9].value;
	Eta = (double *) FieldValueBuild[6].value;
	Data = (unsigned long *) FieldValueBuild[3].value;
	Reddito = (double *) FieldValueBuild[0].value;
	Studente = (char *) FieldValueBuild[7].value;
	Sesso = (char *) FieldValueBuild[4].value;
	Comune = (char *) FieldValueBuild[1].value;
	Provincia = (char *) FieldValueBuild[8].value;
	Telefono = (char *) FieldValueBuild[5].value;
	Indirizzo = (char *) FieldValueBuild[2].value;
	
	// credo che questi valori non servano a nulla
	strcpy(Nome, "MarioRossi");
	*Eta = 10;
	*Data = 777;
	*Reddito = 0.87;
	*Studente = (char) 0;
	*Sesso = 'F';
	strcpy(Comune, "HELLZAPOPPING");
	strcpy(Provincia, "QQ");
	strcpy(Telefono, "00112233");
	strcpy(Indirizzo, "Via Alvise Persichetti 77");
	
//
// Valori per Search
//	
	
	// leggo informazioni campi DBF per ricerca
	ret = GetFieldInfo(handleSearch, 0, &FieldData[0]);
	if (ret != LPC_OK)
		return(ret);
	
	// Inizializzo FieldValueNoIndex (non indicizzato nono campo del DBF - Indirizzo)
	idx = 9;
	strcpy(FieldValueNoIndex.Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueNoIndex.value = calloc(len, sizeof(char));
	strcpy(FieldValueNoIndex.value, "Via_dei_Gerani,_1");

// Inizializzo FieldValueSearchIn, NumOccurrences e OccurrenceIn
// Alla fine NumIn e' pari al numero di elementi da cercare
	
	NumIn = 0;
	
	// Campo Nome
	idx = 0;
	
	strcpy(FieldValueSearchIn[NumIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchIn[NumIn].value = calloc(len, sizeof(char));
	memset(FieldValueSearchIn[NumIn].value, 'A', 127);
	NumOccurrences[NumIn] = 1;
	OccurrenceIn[NumIn][0] = 20;
	NumIn++;
	
	strcpy(FieldValueSearchIn[NumIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchIn[NumIn].value = calloc(len, sizeof(char));
	memset(FieldValueSearchIn[NumIn].value, 'b', 10);
	NumOccurrences[NumIn] = 1;
	OccurrenceIn[NumIn][0] = 30;
	NumIn++;
	
	strcpy(FieldValueSearchIn[NumIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchIn[NumIn].value = calloc(len, sizeof(char));
	strcpy(FieldValueSearchIn[NumIn].value, "SilviaMirisola");
	NumOccurrences[NumIn] = 1;
	OccurrenceIn[NumIn][0] = 100;
	NumIn++;
	
	strcpy(FieldValueSearchIn[NumIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchIn[NumIn].value = calloc(len, sizeof(char));
	strcpy(FieldValueSearchIn[NumIn].value, "AlbertoCangani");
	NumOccurrences[NumIn] = 1;
	OccurrenceIn[NumIn][0] = 1;
	NumIn++;
	
	strcpy(FieldValueSearchIn[NumIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchIn[NumIn].value = calloc(len, sizeof(char));
	strcpy(FieldValueSearchIn[NumIn].value, "EnricoRisa");
	NumOccurrences[NumIn] = 1;
	OccurrenceIn[NumIn][0] = 41;
	NumIn++;
	
	// Campo Eta
	idx = 1;
	
	strcpy(FieldValueSearchIn[NumIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchIn[NumIn].value = calloc(len, sizeof(char));
	Eta = (double *) FieldValueSearchIn[NumIn].value;
	*Eta = 1;
	NumOccurrences[NumIn] = 2;
	OccurrenceIn[NumIn][0] = 14;
	OccurrenceIn[NumIn][1] = 52;
	NumIn++;
	
	strcpy(FieldValueSearchIn[NumIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchIn[NumIn].value = calloc(len, sizeof(char));
	Eta = (double *) FieldValueSearchIn[NumIn].value;
	*Eta = 99;
	NumOccurrences[NumIn] = 1;
	OccurrenceIn[NumIn][0] = 30;
	NumIn++;
	
	strcpy(FieldValueSearchIn[NumIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchIn[NumIn].value = calloc(len, sizeof(char));
	Eta = (double *) FieldValueSearchIn[NumIn].value;
	*Eta = 42;
	NumOccurrences[NumIn] = 1;
	OccurrenceIn[NumIn][0] = 1;
	NumIn++;
	
	strcpy(FieldValueSearchIn[NumIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchIn[NumIn].value = calloc(len, sizeof(char));
	Eta = (double *) FieldValueSearchIn[NumIn].value;
	*Eta = 50;
	NumOccurrences[NumIn] = 2;
	OccurrenceIn[NumIn][0] = 76;
	OccurrenceIn[NumIn][1] = 100;
	NumIn++;
	
	// Campo Data
	idx = 2;
	
	strcpy(FieldValueSearchIn[NumIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchIn[NumIn].value = calloc(len, sizeof(char));
	Data = (unsigned long *) FieldValueSearchIn[NumIn].value;
	*Data = 1234567;
	NumOccurrences[NumIn] = 1;
	OccurrenceIn[NumIn][0] = 20;
	NumIn++;
	
	strcpy(FieldValueSearchIn[NumIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchIn[NumIn].value = calloc(len, sizeof(char));
	Data = (unsigned long *) FieldValueSearchIn[NumIn].value;
	*Data = 0xFE888547;		// 4270359879
	NumOccurrences[NumIn] = 1;
	OccurrenceIn[NumIn][0] = 5;
	NumIn++;
	
	strcpy(FieldValueSearchIn[NumIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchIn[NumIn].value = calloc(len, sizeof(char));
	Data = (unsigned long *) FieldValueSearchIn[NumIn].value;
	*Data = 393765778;
	NumOccurrences[NumIn] = 1;
	OccurrenceIn[NumIn][0] = 1;
	NumIn++;
	
	strcpy(FieldValueSearchIn[NumIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchIn[NumIn].value = calloc(len, sizeof(char));
	Data = (unsigned long *) FieldValueSearchIn[NumIn].value;
	*Data = 784956280;
	NumOccurrences[NumIn] = 1;
	OccurrenceIn[NumIn][0] = 100;
	NumIn++;
	
	// Campo Studente
	idx = 4;
	
	strcpy(FieldValueSearchIn[NumIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchIn[NumIn].value = calloc(len, sizeof(char));
	Studente = (char *) FieldValueSearchIn[NumIn].value;
	*Studente = (char) 0;
	NumOccurrences[NumIn] = 44;
	{
		int Occ[] = {
			2,  4,  5,  6,  9, 13, 14, 15, 16, 17,
			20, 21, 27, 29, 30, 31, 32, 36, 39, 43,
			46, 53, 58, 61, 63, 65, 66, 67, 69, 73,
			75, 81, 82, 85, 87, 89, 90, 91, 92, 93,
			95, 96, 97, 99
		};
		for (i = 0; i < NumOccurrences[NumIn]; i++)
			OccurrenceIn[NumIn][i] = Occ[i]+1;
	}
	NumIn++;
	
	strcpy(FieldValueSearchIn[NumIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchIn[NumIn].value = calloc(len, sizeof(char));
	Studente = (char *) FieldValueSearchIn[NumIn].value;
	*Studente = (char) 1;
	NumOccurrences[NumIn] = 56;
	{
		int Occ[] = {
			 0,  1,  3,  7,  8, 10, 11, 12, 18, 19,
			22, 23, 24, 25, 26, 28, 33, 34, 35, 37,
			38, 40, 41, 42, 44, 45, 47, 48, 49, 50,
			51, 52, 54, 55, 56, 57, 59, 60, 62, 64,
			68, 70, 71, 72, 74, 76, 77, 78, 79, 80,
			83, 84, 86, 88, 94, 98
		};	
		for (i = 0; i < NumOccurrences[NumIn]; i++)
			OccurrenceIn[NumIn][i] = Occ[i]+1;
	}
	NumIn++;

	
// Inizializzo FieldValueSearchNotIn
// Alla fine NumNotIn e' pari al numero di elementi da cercare

	NumNotIn = 0;

	// Campo Nome
	idx = 0;
	
	strcpy(FieldValueSearchNotIn[NumNotIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchNotIn[NumNotIn].value = calloc(len, sizeof(char));
	memset(FieldValueSearchNotIn[NumNotIn].value, 'A', 126);
	NumNotIn++;
	
	strcpy(FieldValueSearchNotIn[NumNotIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchNotIn[NumNotIn].value = calloc(len, sizeof(char));
	strcpy(FieldValueSearchNotIn[NumNotIn].value, "DanieleMoro");
	NumNotIn++;
	
	strcpy(FieldValueSearchNotIn[NumNotIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchNotIn[NumNotIn].value = calloc(len, sizeof(char));
	strcpy(FieldValueSearchNotIn[NumNotIn].value, "ZorroZuzzurro");
	NumNotIn++;
	
	// Campo Eta
	idx = 1;
	
	strcpy(FieldValueSearchNotIn[NumNotIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchNotIn[NumNotIn].value = calloc(len, sizeof(char));
	Eta = (double *) FieldValueSearchNotIn[NumNotIn].value;
	*Eta = 0;
	NumNotIn++;
		
	strcpy(FieldValueSearchNotIn[NumNotIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchNotIn[NumNotIn].value = calloc(len, sizeof(char));
	Eta = (double *) FieldValueSearchNotIn[NumNotIn].value;
	*Eta = 10;
	NumNotIn++;
	
	strcpy(FieldValueSearchNotIn[NumNotIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchNotIn[NumNotIn].value = calloc(len, sizeof(char));
	Eta = (double *) FieldValueSearchNotIn[NumNotIn].value;
	*Eta = 100;
	NumNotIn++;

	// Campo Data
	idx = 2;
	
	strcpy(FieldValueSearchNotIn[NumNotIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchNotIn[NumNotIn].value = calloc(len, sizeof(char));
	Data = (unsigned long *) FieldValueSearchNotIn[NumNotIn].value;
	*Data = 1234566;
	NumNotIn++;
		
	strcpy(FieldValueSearchNotIn[NumNotIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchNotIn[NumNotIn].value = calloc(len, sizeof(char));
	Data = (unsigned long *) FieldValueSearchNotIn[NumNotIn].value;
	*Data = 340835500;
	NumNotIn++;
	
	strcpy(FieldValueSearchNotIn[NumNotIn].Name, FieldData[idx].Name);
	len = FieldData[idx].FieldLen;
	if (FieldData[idx].FieldType == CHARACTER)
		len++;
	FieldValueSearchNotIn[NumNotIn].value = calloc(len, sizeof(char));
	memset(FieldValueSearchNotIn[NumNotIn].value, 0, len);
	Data = (unsigned long *) FieldValueSearchNotIn[NumNotIn].value;
	*Data = 0xFE888558;		// 4270359896 
	NumNotIn++;

//
// Valori per Reindex
//

// leggo informazioni campi DBF per ricerca
	ret = GetFieldInfo(handleReindex[0], 0, &FieldData[0]);
	if (ret != LPC_OK)
		return(ret);

	for (i = 0; i < NUMFIELDS; i++)
		for (j = 0; j < NUM_DEL_ADD; j++)
		{		
			strcpy(FieldValueDelAdd[j][i].Name, FieldData[i].Name);
			len = FieldData[i].FieldLen;
			if (FieldData[i].FieldType == CHARACTER)
				len++;
			FieldValueDelAdd[j][i].value = calloc(len, sizeof(char));
		}
	
	
// RiempioFieldValueDelAdd e PositionDelAdd
	Nome = (char *) FieldValueDelAdd[0][0].value;
	Eta = (double *) FieldValueDelAdd[0][1].value;
	Data = (unsigned long *) FieldValueDelAdd[0][2].value;
	Reddito = (double *) FieldValueDelAdd[0][3].value;
	Studente = (char *) FieldValueDelAdd[0][4].value;
	Sesso = (char *) FieldValueDelAdd[0][5].value;
	Comune = (char *) FieldValueDelAdd[0][6].value;
	Provincia = (char *) FieldValueDelAdd[0][7].value;
	Telefono = (char *) FieldValueDelAdd[0][8].value;
	Indirizzo = (char *) FieldValueDelAdd[0][9].value;
	
	strcpy(Nome, "SilviaMirisola");
	*Eta = 50;
	*Data = 784956280;
	*Reddito = 758.0;
	*Studente = (char) 0;
	*Sesso = 'F';
	strcpy(Comune, "DJ");
	strcpy(Provincia, "OO");
	strcpy(Telefono, "032967");
	strcpy(Indirizzo, "P.ZZA_SAN_LEONE_");

	PositionDelAdd[0] = 100;
	
	Nome = (char *) FieldValueDelAdd[1][0].value;
	Eta = (double *) FieldValueDelAdd[1][1].value;
	Data = (unsigned long *) FieldValueDelAdd[1][2].value;
	Reddito = (double *) FieldValueDelAdd[1][3].value;
	Studente = (char *) FieldValueDelAdd[1][4].value;
	Sesso = (char *) FieldValueDelAdd[1][5].value;
	Comune = (char *) FieldValueDelAdd[1][6].value;
	Provincia = (char *) FieldValueDelAdd[1][7].value;
	Telefono = (char *) FieldValueDelAdd[1][8].value;
	Indirizzo = (char *) FieldValueDelAdd[1][9].value;

	strcpy(Nome, "GianlorenzoBruno");
	*Eta = 85.0;
	*Data = 61549512;
	*Reddito = 1844.0;
	*Studente = (char) 0;
	*Sesso = 'M';
	strcpy(Comune, "EF");
	strcpy(Provincia, "RA");
	strcpy(Telefono, "013428");
	strcpy(Indirizzo, "VIA_SAN_SALVATORE,_PAL.7");
	
	PositionDelAdd[1] = 54;
	
	Nome = (char *) FieldValueDelAdd[2][0].value;
	Eta = (double *) FieldValueDelAdd[2][1].value;
	Data = (unsigned long *) FieldValueDelAdd[2][2].value;
	Reddito = (double *) FieldValueDelAdd[2][3].value;
	Studente = (char *) FieldValueDelAdd[2][4].value;
	Sesso = (char *) FieldValueDelAdd[2][5].value;
	Comune = (char *) FieldValueDelAdd[2][6].value;
	Provincia = (char *) FieldValueDelAdd[2][7].value;
	Telefono = (char *) FieldValueDelAdd[2][8].value;
	Indirizzo = (char *) FieldValueDelAdd[2][9].value;
	
	strcpy(Nome, "FrancoDAgostino");
	*Eta = 33.0;
	*Data = 1010103006;
	*Reddito = 248.0;
	*Studente = (char) 1;
	*Sesso = 'M';
	strcpy(Comune, "FU");
	strcpy(Provincia, "VQ");
	strcpy(Telefono, "024365");
	strcpy(Indirizzo, "VIA_NEPAL,_29");
	
	PositionDelAdd[2] = 50;
	
	Nome = (char *) FieldValueDelAdd[3][0].value;
	Eta = (double *) FieldValueDelAdd[3][1].value;
	Data = (unsigned long *) FieldValueDelAdd[3][2].value;
	Reddito = (double *) FieldValueDelAdd[3][3].value;
	Studente = (char *) FieldValueDelAdd[3][4].value;
	Sesso = (char *) FieldValueDelAdd[3][5].value;
	Comune = (char *) FieldValueDelAdd[3][6].value;
	Provincia = (char *) FieldValueDelAdd[3][7].value;
	Telefono = (char *) FieldValueDelAdd[3][8].value;
	Indirizzo = (char *) FieldValueDelAdd[3][9].value;

	strcpy(Nome, "AlbertoCangani");
	*Eta = 42.0;
	*Data = 393765778;
	*Reddito = 2650.0;
	*Studente = (char) 1;
	*Sesso = 'F';
	strcpy(Comune, "ME");
	strcpy(Provincia, "AY");
	strcpy(Telefono, "033850");
	strcpy(Indirizzo, "VIA_VETRI_VECCHI,_34");
	
	PositionDelAdd[3] = 1;

	return(ret);
}


/*****************
* QUIT
*****************/
void Quit(void)
{
	int i;

	(void) CloseDatabaseFile(handleBuildIndex);
	(void) CloseDatabaseFile(handleSearch);
	for (i  = 0; i <= NUM_DEL_ADD; i++)
		(void) CloseDatabaseFile(handleReindex[i]);

	for (i = 0; i < NUMFIELDS; i++)
	{
		free(FieldValueBuild[i].value);
		
		/* completare con pulizia dati ancora non inseriti */
		
/*		
		free(FieldValueAll[i].value);
		free(Record20[i].value);
		free(Record30[i].value);
*/	
	}

}

/*********************
* CONFRONTAVALUES
**********************/
int ConfrontaValues(FIELD_VALUE_t *FieldValue, FIELD_VALUE_t *FieldValueRef, int NumValues)
{
	int i, j, k;

	for (i = 0; i < NumValues; i++)
	{
		for (j = 0; j < NUMFIELDS; j++)
			if (!strcmp(FieldValue[i].Name, FieldData[j].Name))
				break; 
		for (k = 0; k < NUMFIELDS; k++)
			if (!strcmp(FieldValue[i].Name, FieldValueRef[k].Name))
				break; 
		if (memcmp(FieldValue[i].value, FieldValueRef[k].value, FieldData[j].FieldLen))
			break;
	}

	if (i < NumValues)
		return(-1);

	return(0);
}

/*! \} */	// Fine gruppo TestModulo4
/*! \} */	// Fine gruppo Modulo4
